#include <WiFi.h>
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_Client.h"
#include <Adafruit_SSD1306.h>

const char* ssid = "TECNO SPARK Go 2020";
const char* password = "@felicalon";

#define AIO_SERVER      "io.adafruit.com"
#define AIO_SERVERPORT  1883
#define AIO_USERNAME    "felicalon"
#define AIO_KEY         "aio_yJLL11JJPvPxwBbEFXK8i3TalJpx"

WiFiClient client;
Adafruit_MQTT_Client mqtt(&client, AIO_SERVER, AIO_SERVERPORT, AIO_USERNAME, AIO_KEY);

Adafruit_MQTT_Publish feed = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/moisture");
const int moisturePin = 15;  
const int moistureThreshold = 5000;

#define OLED_RESET     -1
Adafruit_SSD1306 display(OLED_RESET);

void setup() {
    Serial.begin(115200);

    WiFi.begin(ssid, password);
    while (WiFi.status() != WL_CONNECTED) {
        delay(500);
        Serial.print(".");
    }
    Serial.println("WiFi connected");

    mqtt.connect();
    while (!mqtt.connected()) {
        Serial.print(".");
        delay(1000);
    }
    Serial.println("MQTT connected");

    display.begin(SSD1306_SWITCHCAPVCC, 0x3C, true, false);
    display.display();
    delay(2000);
    display.clearDisplay();
}

void loop() {
    int moistureValue = analogRead(moisturePin);
    float voltage = (moistureValue / 4095.0) * 3.3; // Convert to voltage (assuming 12-bit ADC)

    // Display moisture value on OLED
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(0, 0);
    display.println("Moisture:");
    display.setCursor(0, 12);
    display.println(voltage, 2); // Display voltage with 2 decimal places
    display.display();

    // Display moisture value on Serial Monitor
    Serial.print("Moisture: ");
    Serial.print(voltage, 2);
    Serial.println("V");

    // Check moisture threshold and send notification
    if (voltage <  moistureThreshold) {
        feed.publish(voltage);
    }

    delay(4000);
}

void MQTT_connect() {
  int8_t ret;

  // Stop if already connected.
  if (mqtt.connected()) {
    return;
  }

  Serial.print("Connecting to MQTT... ");

  uint8_t retries = 3;
  while ((ret = mqtt.connect()) != 0) { // connect will return 0 for connected
       Serial.println(mqtt.connectErrorString(ret));
       Serial.println("Retrying MQTT connection in 5 seconds...");
       mqtt.disconnect();
       delay(5000);  // wait 5 seconds
       retries--;
       if (retries == 0) {
         // basically die and wait for WDT to reset me
         while (1);
       }
  }
  Serial.println("MQTT Connected!");
}

 
